@@ -1,8 +1,8 @@
+# -*- coding: utf-8 -*-
 import collections
 from typing import Optional
 import typing
 
-import mincepy
 from . import depositors
 from . import refs
 from . import types
@@ -108,14 +108,14 @@ class ConvenienceMixin:
     def set_meta(self, meta: Optional[dict]):
         """Set the metadata dictionary for this object"""
         if self._historian is None:
-            raise RuntimeError("Object must be saved before the metadata can be set")
+            raise RuntimeError('Object must be saved before the metadata can be set')
 
         self._historian.meta.set(self, meta)
 
     def update_meta(self, meta: dict):
         """Update the metadata dictionary for this object"""
         if self._historian is None:
-            raise RuntimeError("Object must be saved before the metadata can be updated")
+            raise RuntimeError('Object must be saved before the metadata can be updated')
 
         self._historian.meta.update(self, meta)
 
@@ -128,7 +128,11 @@ class ConvenienceMixin:
 
     def save(self, meta: dict = None):
         """Save the object"""
-        historian = self._historian or mincepy.get_historian()
+        if self._historian is None:
+            from . import history
+            historian = history.get_historian()
+        else:
+            historian = self._historian
         return historian.save_one(self, meta=meta)
 
     def sync(self):
@@ -140,7 +144,7 @@ class ConvenienceMixin:
         self._on_save(saver)
         return super().save_instance_state(saver)
 
-    def load_instance_state(self, saved_state, loader: 'mincepy.Loader'):
+    def load_instance_state(self, saved_state, loader: 'depositors.Loader'):
         """Take the given object and load the instance state into it"""
         super().load_instance_state(saved_state, loader)
         self._on_load(loader)
