@@ -10,8 +10,7 @@ except ImportError:
 import getpass
 import logging
 import socket
-import typing
-from typing import MutableMapping, Any, Optional, Iterable, Union, Iterator, Type, Dict, Callable
+from typing import MutableMapping, Any, Optional, Iterable, Union, Iterator, Type, Dict, Callable, Sequence
 import weakref
 
 import deprecation
@@ -46,7 +45,7 @@ __all__ = 'Historian', 'ObjectEntry'
 logger = logging.getLogger(__name__)
 
 ObjectEntry = collections.namedtuple('ObjectEntry', 'ref obj')
-HistorianType = Union[helpers.TypeHelper, typing.Type[types.SavableObject]]
+HistorianType = Union[helpers.TypeHelper, Type[types.SavableObject]]
 
 
 class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-attributes
@@ -103,8 +102,6 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
 
         self._transactions = None
 
-        self._saving_set = set()
-
         self._user = getpass.getuser()
         self._hostname = socket.gethostname()
 
@@ -113,18 +110,8 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         self._migrate = migrate.Migrations(self)
         self._references = hist.References(self)
 
-        self._snapshots = frontend.ObjectCollection(
-            self,
-            self._archive.snapshots,
-            record_factory=lambda record_dict: SnapshotLoadableRecord(
-                record_dict, self.load_snapshot_from_record),
-            obj_loader=self.load_snapshot_from_record)
-        self._objects = frontend.ObjectCollection(
-            self,
-            self._archive.objects,
-            record_factory=lambda record_dict: LoadableRecord(
-                record_dict, self.load_snapshot_from_record, self._load_object_from_record),
-            obj_loader=self._load_object_from_record)
+        self._snapshots = hist.SnapshotsCollection(self, self._archive.snapshots)
+        self._objects = hist.LiveObjectsCollection(self, self._archive.objects)
 
     @property
     def archive(self):
@@ -146,22 +133,22 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         return self._migrate
 
     @property
-    def records(self) -> frontend.EntriesCollection['LoadableRecord']:
+    def records(self) -> frontend.EntriesCollection[recordsm.DataRecord]:
         """Access methods and properties that act on and return data records"""
         return self._objects.records
 
     @property
-    def objects(self) -> frontend.ObjectCollection:
+    def objects(self) -> hist.LiveObjectsCollection:
         """Access the snapshots"""
         return self._objects
 
     @property
     def references(self) -> hist.References:
-        """Access the references possibilities"""
+        """Access the references collection"""
         return self._references
 
     @property
-    def snapshots(self) -> frontend.ObjectCollection:
+    def snapshots(self) -> hist.SnapshotsCollection:
         """Access the snapshots"""
         return self._snapshots
 
@@ -183,7 +170,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
             if isinstance(entry, tuple):
                 if len(entry) > 2:
                     raise ValueError(
-                        "Supplied tuples can only contain (object, meta), got '{}'".format(entry))
+                        f"Supplied tuples can only contain (object, meta), got '{entry}'")
             else:
                 entry = (entry,)
             to_save.append(entry)
@@ -215,7 +202,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
 
         # Save the object and metadata
         with self.in_transaction():
-            record = self._save_object(obj, self._live_depositor)
+            record = self._live_depositor._save_object(obj)  # pylint: disable=protected-access
             if meta:
                 self.meta.update(record.obj_id, meta)
 
@@ -266,12 +253,20 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
 
         return loaded
 
-    def load_one(self, obj_id_or_snapshot_id):
+    def load_one(self, obj_id_or_snapshot_id) -> object:
         """Load one object or snapshot from the database"""
         if isinstance(obj_id_or_snapshot_id, recordsm.SnapshotId):
             return self.load_snapshot(obj_id_or_snapshot_id)
 
-        return self._load_object(obj_id_or_snapshot_id, self._live_depositor)
+        # OK, assume we're dealing with an object id
+        obj_id = self._ensure_obj_id(obj_id_or_snapshot_id)
+
+        # Try getting the object from the our dict of up to date ones
+        try:
+            return self.get_obj(obj_id)
+        except exceptions.NotFound:
+            # Going to have to load from the database
+            return self._live_depositor._load_object(obj_id)  # pylint: disable=protected-access
 
     def get(self, obj_id) -> object:
         """Get a live object using the object id"""
@@ -292,7 +287,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         record = self._objects.records.get(obj_id)
 
         if record.is_deleted_record():
-            raise exceptions.ObjectDeleted("Object with id '{}' has been deleted".format(obj_id))
+            raise exceptions.ObjectDeleted(f"Object with id '{obj_id}' has been deleted")
 
         if record.version == self.get_snapshot_id(obj).version:
             # Nothing has changed
@@ -302,7 +297,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         return self._live_depositor.update_from_record(obj, record)
 
     def delete(self, *obj_or_identifier, imperative=True) -> result_types.DeleteResult:
-        """Delete an object.
+        """Delete objects.
 
         :param imperative: if True, this means that the caller explicitly expects this call to delete the passed objects
             and it should therefore raise if an object cannot be found or has been deleted already.  If False, the
@@ -313,7 +308,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         # We need the current records to be able to build the delete records
         obj_ids = list(map(self._ensure_obj_id, obj_or_identifier))
 
-        # Find the current records
+        # Find the current records (i.e. from our cache)
         records = {}  # type: Dict[Any, recordsm.DataRecord]
         left_to_find = set()
         for obj_id in obj_ids:
@@ -334,6 +329,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
                 left_to_find.remove(record.obj_id)
 
         if left_to_find and imperative:
+            # Still couldn't find them so raise
             raise exceptions.NotFound(left_to_find)
 
         deleted = []
@@ -353,12 +349,10 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
 
         return result_types.DeleteResult(deleted, left_to_find)
 
-    def history(
-            self,
-            obj_or_obj_id,
-            idx_or_slice='*',
-            as_objects=True
-    ) -> [typing.Sequence[ObjectEntry], typing.Sequence[recordsm.DataRecord]]:
+    def history(self,
+                obj_or_obj_id,
+                idx_or_slice='*',
+                as_objects=True) -> [Sequence[ObjectEntry], Sequence[recordsm.DataRecord]]:
         """Get a sequence of object ids and instances from the history of the given object.
 
         :param obj_or_obj_id: The instance or id of the object to get the history for
@@ -474,13 +468,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
             # Maybe we've been passed an object
             pass
 
-        try:
-            return self.get_obj_id(obj_or_identifier)
-        except exceptions.NotFound:
-            pass
-
-        # Out of options
-        return None
+        return self.get_obj_id(obj_or_identifier)
 
     def get_snapshot_id(self, obj: object) -> recordsm.SnapshotId:
         """Get the current snapshot id for a live object.  Will return the id or raise
@@ -543,8 +531,9 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         return self.get_helper(type_id).TYPE
 
     def get_helper(self, type_id_or_type, auto_register=False) -> helpers.TypeHelper:
-        if auto_register and issubclass(type_id_or_type, types.SavableObject):
-            self._ensure_compatible(type_id_or_type)
+        if auto_register and issubclass(type_id_or_type, types.SavableObject) and \
+                type_id_or_type not in self._type_registry:
+            self.register_type(type_id_or_type)
 
         return self._type_registry.get_helper(type_id_or_type)
 
@@ -668,7 +657,8 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         # REMOTE
         remote = result_set.historian  # type: Historian
         # Get information about the records that we've been asked to merge
-        remote_partial_records = result_set._project(recordsm.OBJ_ID, recordsm.VERSION)  # pylint: disable=protected-access
+        # pylint: disable=protected-access
+        remote_partial_records = result_set._project(recordsm.OBJ_ID, recordsm.VERSION)
         remote_snapshot_ids = set(map(recordsm.SnapshotId.from_dict,
                                       remote_partial_records))  # DB HIT
 
@@ -702,6 +692,38 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
 
         return result
 
+    def purge(self, deleted=True, unreferenced=True, dry_run=True) -> result_types.PurgeResult:
+        """Purge the archive of unused snapshots"""
+        snapshot_purge = self.snapshots.purge(deleted=deleted, dry_run=dry_run)
+
+        unreferenced_deleted = set()
+        if unreferenced:
+            # Let's get snapshot ids for all live object
+            live_snapshot_ids = list(
+                map(
+                    recordsm.SnapshotId.from_dict,
+                    # pylint: disable=protected-access
+                    self.objects.records.find()._project(recordsm.OBJ_ID, recordsm.VERSION)))
+            # Now, find all the snapshots that they refer to, these will be the ones we DON'T delete
+            reg_graph = self.references.get_snapshot_ref_graph(*live_snapshot_ids,
+                                                               direction=archives.OUTGOING,
+                                                               max_dist=-1)  # DB HIT
+            to_keep = set(reg_graph.nodes)
+
+            res = self.snapshots.records.find(
+                expr.Comparison('_id', expr.Nin(list(map(str, to_keep)))))
+
+            unreferenced_deleted = set(
+                map(
+                    recordsm.SnapshotId.from_dict,
+                    # pylint: disable=protected-access
+                    res._project(recordsm.OBJ_ID, recordsm.VERSION)))
+
+            if unreferenced_deleted and not dry_run:
+                self._archive.bulk_write(list(map(operations.Delete, unreferenced_deleted)))
+
+        return result_types.PurgeResult(snapshot_purge.deleted_purged, unreferenced_deleted)
+
     def _merge_batch(self, remote: 'Historian',
                      remote_ref_graph: networkx.DiGraph) -> result_types.MergeResult:
         sid_strings = list(map(str, remote_ref_graph.nodes))
@@ -771,7 +793,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
                                         merged_snapshots=remote_partial_records.keys())
 
     @contextlib.contextmanager
-    def in_transaction(self):
+    def in_transaction(self) -> Iterator[Transaction]:
         """This context will either re-use an existing transaction, if one is currently taking place
         or create a new one if not."""
         current = self.current_transaction()
@@ -784,7 +806,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
             yield trans
 
     @contextlib.contextmanager
-    def transaction(self):
+    def transaction(self) -> Iterator[Transaction]:
         """Start a new transaction.  Will be nested if there is already one underway"""
         if self._transactions:
             # Start a nested one
@@ -866,11 +888,8 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         if trans.metas:
             self._archive.meta_set_many(trans.metas)
 
-    def _load_object_from_record(self,
-                                 record: recordsm.DataRecord,
-                                 depositor: depositors.LiveDepositor = None):
-
-        depositor = depositor or self._live_depositor
+    def _load_object_from_record(self, record: recordsm.DataRecord):
+        depositor = self._live_depositor
 
         # Try getting the object from the our dict of up to date ones
         obj_id = record.obj_id
@@ -891,98 +910,6 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
             # Ok, just use the one from the archive
             return depositor.load_from_record(record)
 
-    def _load_object(self, obj_id, depositor: depositors.LiveDepositor):
-        obj_id = self._ensure_obj_id(obj_id)
-
-        # Try getting the object from the our dict of up to date ones
-        try:
-            return self.get_obj(obj_id)
-        except exceptions.NotFound:
-            pass
-
-        with self.in_transaction() as trans:
-            if trans.is_deleted(obj_id):
-                raise exceptions.ObjectDeleted(obj_id)
-
-            record = self._objects.records.get(obj_id)
-
-            if record.is_deleted_record():
-                raise exceptions.ObjectDeleted(obj_id)
-
-            try:
-                obj = self._live_objects.get_object(obj_id)
-            except exceptions.NotFound:
-                logger.debug('Loading object from record: %s', record.snapshot_id)
-                # Ok, just use the one from the archive
-                return depositor.load_from_record(record)
-            else:
-                if record.version != self.get_snapshot_id(obj).version:
-                    # The one in the archive is newer, so use that
-                    logger.debug('Updating object from record: %s', record.snapshot_id)
-                    depositor.update_from_record(obj, record)
-
-                return obj
-
-    def _save_object(self, obj, depositor) -> recordsm.DataRecord:
-        with self.in_transaction() as trans:
-            try:
-                helper = self._ensure_compatible(type(obj))
-            except TypeError:
-                raise TypeError('Type is incompatible with the historian: {}'.format(
-                    type(obj).__name__)) from None
-
-            # Check if an object is already being saved in the transaction
-            try:
-                record = trans.get_record_for_live_object(obj)
-                return record
-            except exceptions.NotFound:
-                pass
-
-            with self._saving(obj):
-                # Ok, have to save it
-                current_hash = self.hash(obj)
-
-                try:
-                    # Let's see if we have a record at all
-                    record = self._live_objects.get_record(obj)
-                except exceptions.NotFound:
-                    # Object being saved for the first time
-                    builder = self._create_builder(obj, snapshot_hash=current_hash)
-                    record = depositor.save_from_builder(obj, builder)
-                    if self.meta.sticky:
-                        # Apply the sticky meta
-                        self.meta.update(record.obj_id, self.meta.sticky)
-                    return record
-                else:
-                    if helper.IMMUTABLE:
-                        logger.info("Tried to save immutable object with id '%s' again",
-                                    record.obj_id)
-                        return record
-
-                    # Check if our record is up to date
-                    with self.transaction() as nested:
-                        loaded_obj = self._new_snapshot_depositor().load_from_record(record)
-
-                        if current_hash == record.snapshot_hash and self.eq(obj, loaded_obj):
-                            # Objects identical
-                            nested.rollback()
-                        else:
-                            builder = record.child_builder(snapshot_hash=current_hash)
-                            record = depositor.save_from_builder(obj, builder)
-
-                    return record
-
-    def _create_builder(self, obj, **additional) -> recordsm.DataRecordBuilder:
-        """Create a record builder for a new object object"""
-        additional = additional or {}
-        helper = self._ensure_compatible(type(obj))
-
-        builder = recordsm.DataRecord.new_builder(type_id=helper.TYPE_ID,
-                                                  obj_id=self._archive.create_archive_id(),
-                                                  version=0)
-        builder.update(additional)
-        return builder
-
     def _ensure_obj_id(self, obj_or_identifier):
         """
         This call will try and get an object id from the passed parameter.  Uses .to_obj_id() and raises NotFound if it
@@ -990,17 +917,10 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         """
         obj_id = self.to_obj_id(obj_or_identifier)
         if obj_id is None:
-            raise exceptions.NotFound(
-                "Could not get an object id from '{}'".format(obj_or_identifier))
+            raise exceptions.NotFound(f"Could not get an object id from '{obj_or_identifier}'")
 
         return obj_id
 
-    def _ensure_compatible(self, obj_type: Type[types.SavableObject]) -> helpers.TypeHelper:
-        if obj_type not in self._type_registry:
-            self.register_type(obj_type)
-
-        return self._type_registry.get_helper_from_obj_type(obj_type)
-
     def _prepare_obj_id(self, obj_id):
         if obj_id is None:
             return None
@@ -1011,7 +931,7 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         except exceptions.NotFound as exc:
             # Maybe it is multiple object ids
             if not isinstance(obj_id, Iterable):  # pylint: disable=isinstance-second-argument-not-valid-type
-                raise TypeError("Cannot get object id(s) from '{}'".format(obj_id)) from exc
+                raise TypeError(f"Cannot get object id(s) from '{obj_id}'") from exc
 
             return list(map(self._ensure_obj_id, obj_id))
 
@@ -1024,26 +944,10 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
         except TypeError as exc:
             # Maybe it is multiple type ids
             if not isinstance(obj_type, Iterable):  # pylint: disable=isinstance-second-argument-not-valid-type
-                raise TypeError("Cannot get type id(s) from '{}'".format(obj_type)) from exc
+                raise TypeError(f"Cannot get type id(s) from '{obj_type}'") from exc
 
             return list(map(self.get_obj_type_id, obj_type))
 
-    def _create_loadable_record(self, record: recordsm.DataRecord) -> 'LoadableRecord':
-        return LoadableRecord(record, self.load_snapshot_from_record, self._load_object_from_record)
-
-    @contextlib.contextmanager
-    def _saving(self, obj):
-        obj_id = id(obj)
-        if obj_id in self._saving_set:
-            raise RuntimeError(
-                'The object is already being saved, this cannot be called twice and suggests '
-                'a circular reference is being made')
-        self._saving_set.add(obj_id)
-        try:
-            yield
-        finally:
-            self._saving_set.remove(obj_id)
-
     def _record_builder_created(self,
                                 builder: recordsm.DataRecordBuilder) -> recordsm.DataRecordBuilder:
         """Update a data record builder with standard information."""
@@ -1052,34 +956,3 @@ class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-a
 
     def _new_snapshot_depositor(self):
         return depositors.SnapshotLoader(self)
-
-
-class SnapshotLoadableRecord(recordsm.DataRecord):
-    __slots__ = ()
-
-    def __new__(cls, record_dict: dict, snapshot_loader: Callable[[recordsm.DataRecord], object]):
-        loadable = super().__new__(cls, **record_dict)
-        loadable._snapshot_loader = snapshot_loader
-        return loadable
-
-    def load(self) -> object:
-        return self._snapshot_loader(self)
-
-
-class LoadableRecord(recordsm.DataRecord):
-    __slots__ = ()
-    _obj_loader = None  # type: Optional[Callable[[recordsm.DataRecord], object]]
-    _snapshot_loader = None  # type: Optional[Callable[[recordsm.DataRecord], object]]
-
-    def __new__(cls, record_dict: dict, snapshot_loader: Callable[[recordsm.DataRecord], object],
-                obj_loader: Callable[[recordsm.DataRecord], object]):
-        loadable = super().__new__(cls, **record_dict)
-        loadable._obj_loader = obj_loader
-        loadable._snapshot_loader = snapshot_loader
-        return loadable
-
-    def load_snapshot(self) -> object:
-        return self._snapshot_loader(self)  # pylint: disable=not-callable
-
-    def load(self) -> object:
-        return self._obj_loader(self)  # pylint: disable=not-callable
